local _, RS = ...
RS.Events = RS.Events or {}
local Events = RS.Events

function Events:OnLoad()
	self:OnLoadChat()
	self:OnLoadCombat()
	self:OnLoadMiscellaneous()
	self:OnLoadTracking()
end

----------------------------------------------------------------------------------------------------
-- Event registration
----------------------------------------------------------------------------------------------------
local eventList = {
	"AUCTION_HOUSE_CLOSED",
	"AUCTION_HOUSE_SHOW",
	"AUTOFOLLOW_BEGIN",
	"AUTOFOLLOW_END",
	"BANKFRAME_CLOSED",
	"BANKFRAME_OPENED",
	"COMBAT_LOG_EVENT_UNFILTERED",
	"DELETE_ITEM_CONFIRM",
	"DUEL_REQUESTED",
	"GROUP_ROSTER_UPDATE",
	"LOOT_CLOSED",
	"LOOT_OPENED",
	"MAIL_CLOSED",
	"MAIL_SHOW",
	"MERCHANT_CLOSED",
	"MERCHANT_SHOW",
	"MINIMAP_PING",
	"MIRROR_TIMER_START", -- timer bar starts for breath/fatigue/etc
	"MIRROR_TIMER_STOP",  -- timer bar ends for breath/fatigue/etc
	"PLAYER_ALIVE",
	"PLAYER_CAMPING",
	"PLAYER_QUITING",
	"PLAYER_CONTROL_GAINED",
	"PLAYER_CONTROL_LOST",
	"PLAYER_DEAD",
	"PLAYER_ENTERING_WORLD", -- for SummonAcceptAfter
	"PLAYER_LEAVING_WORLD",  -- for SummonAcceptAfter
	"PLAYER_LEVEL_UP",
	"PLAYER_REGEN_DISABLED", -- entering combat
	"PLAYER_REGEN_ENABLED",  -- exiting combat
	"PLAYER_TARGET_CHANGED",
	"PLAYER_UNGHOST",
	"PLAYER_UPDATE_RESTING",
	"QUEST_ACCEPTED",
	"QUEST_TURNED_IN",
	"READY_CHECK",
	"READY_CHECK_CONFIRM",
	"TAXIMAP_CLOSED",
	"TAXIMAP_OPENED",
	--"TRADE_ACCEPT_UPDATE", -- handled by chat messages instead
	--"TRADE_CLOSED",        -- handled by chat messages instead
	"TRADE_SHOW",
	"UNIT_HEALTH",
	"UNIT_POWER_UPDATE",
	"UNIT_SPELLCAST_CHANNEL_START",
	"UNIT_SPELLCAST_CHANNEL_STOP",
	"UNIT_SPELLCAST_FAILED",
	"UNIT_SPELLCAST_INTERRUPTED",
	"UNIT_SPELLCAST_SENT", -- for saving the target name since SPELLCAST_START won't give it
	"UNIT_SPELLCAST_START",
	"UNIT_SPELLCAST_STOP",
	"UNIT_SPELLCAST_SUCCEEDED",
	"UPDATE_INVENTORY_DURABILITY",
	"ZONE_CHANGED",
	"ZONE_CHANGED_INDOORS",
	"ZONE_CHANGED_NEW_AREA",

	"CHAT_MSG_CHANNEL",
	"CHAT_MSG_EMOTE",
	"CHAT_MSG_GUILD",
	"CHAT_MSG_IGNORED",
	"CHAT_MSG_INSTANCE_CHAT_LEADER",
	"CHAT_MSG_INSTANCE_CHAT",
	"CHAT_MSG_LOOT",
	"CHAT_MSG_MONEY",
	"CHAT_MSG_MONSTER_EMOTE",
	"CHAT_MSG_MONSTER_SAY",
	"CHAT_MSG_MONSTER_YELL",
	"CHAT_MSG_OFFICER",
	"CHAT_MSG_OPENING",
	"CHAT_MSG_PARTY",
	"CHAT_MSG_PARTY_LEADER",
	"CHAT_MSG_RAID_LEADER",
	"CHAT_MSG_RAID_WARNING",
	"CHAT_MSG_RAID",
	"CHAT_MSG_SAY",
	"CHAT_MSG_SYSTEM",
	"CHAT_MSG_TEXT_EMOTE",
	"CHAT_MSG_TRADESKILLS",
	--"CHAT_MSG_WHISPER_INFORM",
	"CHAT_MSG_WHISPER",
	"CHAT_MSG_YELL",
	"UI_ERROR_MESSAGE",
	"UI_INFO_MESSAGE",
}

Events.eventFrame = CreateFrame("frame")
Events.eventFrame:Hide()
Events.eventFrame:SetScript("OnEvent", function(self, event, ...)
	Events[event](Events, ...)
end)

function Events:RegisterReactionsEvents()
	self.eventFrame:Show()
	for i=1,#eventList do
		self.eventFrame:RegisterEvent(eventList[i])
	end
end

function Events:UnregisterReactionsEvents()
	self.eventFrame:Hide()
	for i=1,#eventList do
		self.eventFrame:RegisterEvent(eventList[i])
	end
end

----------------------------------------------------------------------------------------------------
-- Data for handling events
----------------------------------------------------------------------------------------------------
Events.lastZone              = nil   -- the last non-blank zone the player has moved into and reacted to
Events.lastSubzone           = nil   -- the last non-blank subzone the player has moved into and reacted to
Events.onLowHealth           = nil   -- if low health has been reached, this is the lowest seen so far
Events.onLowMana             = nil   -- if low mana has been reached, this is the lowest seen so far
Events.lowestDurability      = 100   -- tracks the lowest durability percentage so events are only used once
Events.autoLootOrTime        = nil   -- 1 if autoloot was used, else the GetTime() the last loot openend event happened
Events.usedStartCast         = nil   -- if the current spell has tried a "You begin casting" action
Events.isDead                = nil   -- if the player is currently dead
Events.followingPlayer       = nil   -- the name of the character the player is following
Events.acceptSummonTime      = 0     -- used with zone changes to know when the player is summoned
Events.pendingGroupJoinName  = nil   -- the name of someone joining the group (only if the chat message comes before them actually joining)
Events.pendingGroupLeaveName = nil   -- the name of someone leaving the group (only if the chat message comes before they actually leave)
Events.sourceFlags           = nil   -- the sourceFlags in the last combat event
Events.sourceName            = nil   -- the sourceName in the last combat event
Events.destFlags             = nil   -- the destFlags in the last combat event
Events.destName              = nil   -- the destName in the last combat event

-- variables to keep track of when something is open because they have 2 closing events
Events.auctioneerName        = nil   -- also the name of the auctioneer
Events.mailboxOpened         = nil   -- if the mailbox is opened
Events.merchantName          = nil   -- also the name of the merchant
Events.tradePartner          = nil   -- also the name of the player trading with
Events.flightMasterOpened    = nil   -- also the name of the flight master

-- localized spell names
Events.DRINK             = (GetSpellInfo(430))
Events.FOOD              = (GetSpellInfo(433))
-- Events.OPENING           = (GetSpellInfo(3365))
Events.WEAK_ALCOHOL      = (GetSpellInfo(11007))
Events.STANDARD_ALCOHOL  = (GetSpellInfo(11008))
Events.STRONG_ALCOHOL    = (GetSpellInfo(11009))
Events.POTENT_ALCOHOL    = (GetSpellInfo(11629))
Events.MOONGLOW_ALCOHOL  = (GetSpellInfo(26389))
Events.DARK_IRON_ALCOHOL = (GetSpellInfo(44540))
