local _, RS = ...
RS.Events = RS.Events or {}
local Events = RS.Events

-- local references - WoW and Lua
local time = time

-- local references - Addon
local groupList = nil
local chatList  = nil
local triggerList = nil
local channelData = nil
local Variables = nil
local playerName = UnitName("player")
local AttemptReaction = nil

function Events:OnLoadChat()
	local addonSettings = ReactionsSave
	groupList = addonSettings.groupList
	chatList  = addonSettings.chatList
	triggerList = chatList.trigger
	Variables = RS.Variables
	AttemptReaction = RS.Reactions.AttemptReaction

	channelData = {
	--  event chatType                     usable name      slash command        reference to its chat trigger list
		["CHAT_MSG_CHANNEL"]              = {"channel",      nil,                 chatList.channel.Channel},
		["CHAT_MSG_EMOTE"]                = {"emote",        SLASH_EMOTE1,        chatList.channel.Emote},
		["CHAT_MSG_GUILD"]                = {"guild",        SLASH_GUILD1,        chatList.channel.Guild},
		["CHAT_MSG_IGNORED"]              = {"system",       nil,                 chatList.channel.System},
		["CHAT_MSG_INSTANCE_CHAT_LEADER"] = {"instance",     SLASH_BATTLEGROUND1, chatList.channel.Instance},
		["CHAT_MSG_INSTANCE_CHAT"]        = {"instance",     SLASH_BATTLEGROUND1, chatList.channel.Instance},
		["CHAT_MSG_LOOT"]                 = {"loot",         nil,                 chatList.channel.Loot},
		["CHAT_MSG_MONEY"]                = {"money",        nil,                 chatList.channel.Loot},
		["CHAT_MSG_MONSTER_EMOTE"]        = {"emote",        SLASH_EMOTE1,        chatList.channel.Emote},
		["CHAT_MSG_MONSTER_SAY"]          = {"say",          SLASH_SAY1,          chatList.channel.Say},
		["CHAT_MSG_MONSTER_YELL"]         = {"yell",         SLASH_YELL1,         chatList.channel.Yell},
		["CHAT_MSG_OFFICER"]              = {"officer",      SLASH_OFFICER1,      chatList.channel.Officer},
		["CHAT_MSG_OPENING"]              = {"system",       nil,                 chatList.channel.System},
		["CHAT_MSG_PARTY_LEADER"]         = {"party",        SLASH_PARTY1,        chatList.channel.Party},
		["CHAT_MSG_PARTY"]                = {"party",        SLASH_PARTY1,        chatList.channel.Party},
		["CHAT_MSG_RAID_LEADER"]          = {"raid",         SLASH_RAID1,         chatList.channel.Raid},
		["CHAT_MSG_RAID_WARNING"]         = {"raid",         SLASH_RAID1,         chatList.channel.Raid},
		["CHAT_MSG_RAID"]                 = {"raid",         SLASH_RAID1,         chatList.channel.Raid},
		["CHAT_MSG_SAY"]                  = {"say",          SLASH_SAY1,          chatList.channel.Say},
		["CHAT_MSG_SYSTEM"]               = {"system",       nil,                 chatList.channel.System},
		["CHAT_MSG_TEXT_EMOTE"]           = {"emote action", nil,                 chatList.channel.Action},
		["CHAT_MSG_TRADESKILLS"]          = {"tradeskill",   nil,                 chatList.channel.Tradeskill},
		["CHAT_MSG_WHISPER"]              = {"whisper",      SLASH_WHISPER1,      chatList.channel.Whisper},
		--["CHAT_MSG_WHISPER_INFORM"]     = {"whisper",      nil,                 chatList.channel.Whisper},
		["CHAT_MSG_YELL"]                 = {"yell",         SLASH_YELL1,         chatList.channel.Yell},
		["UI_ERROR_MESSAGE"]              = {"error",        nil,                 chatList.channel.Error},
		["UI_INFO_MESSAGE"]               = {"error",        nil,                 chatList.channel.Error},
	}
end

-- global table to use in chat trigger custom lua scripts - probably better than setfenv or the annoyance of having to get values from ...
rs = {} -- something short is wanted and it doesn't seem to be used by anything popular
rs.capture = {}
local customLuaTable = rs
local customLuaCapture = customLuaTable.capture

function Events:CheckChatTriggers(chatType, message, author, language, channelDescription)
	local Variables = Variables
	Variables:ClearNames()

	local channelInformation = channelData[chatType]
	if not channelInformation or not message or message == "" then
		return
	end

	if type(message) ~= "string" then
		message = tostring(message)
	end

	author = author and author:gsub("%-.+", "")

	local channelTriggers = channelInformation[3]
	local chatChannel = channelInformation[1]
	local chatChannelNumber
	if chatChannel == "channel" then
		chatChannelNumber, chatChannel = channelDescription:match("^(%d+)%. (.+)")
	else
		chatChannelNumber = nil
	end

	Variables.chatChannelNumber = chatChannelNumber
	Variables.chatChannel = chatChannel
	Variables.chatMessage = message
	Variables.targetName = author

	local trigger
	local triggerName
	local lowercaseMessage = message:gsub("^%s*(.-)%s*$", "%1"):lower()
	local isFriend      = nil -- nil if not checked, true/false if the target was/wasn't on the friends list
	local isGuildMember = nil -- nil if not checked, true/false if the target was/wasn't in the same guild
	for i=1,#channelTriggers do
		repeat -- to be able to use "break" like "continue"
			triggerName = channelTriggers[i]
			trigger = triggerList[triggerName]
			if not trigger then
				break
			end

			-- check if it's yourself
			if (author == playerName and not trigger.matchYourself) then
				break
			end

			-- check if it's in a group that's disabled
			if (trigger.group and not groupList[trigger.group]) or (not trigger.group and not groupList["Ungrouped"]) then
				break
			end

			-- check chance
			if trigger.chance ~= 100 and trigger.chance < random() * 100 then
				break
			end

			-- check cooldown time
			if trigger.globalCooldown and trigger.lastUsed and trigger.lastUsed > time() - trigger.globalCooldown then
				break
			end
			if trigger.personCooldown and trigger.personCooldownList and trigger.personCooldownList[author]
			 and trigger.personCooldownList[author] > time() - trigger.personCooldown then
				break
			end

			-- check if they're allowed to have matches
			if author ~= ""
			 and (not trigger.matchGuild or not trigger.matchOthers or not trigger.matchFriends)
			 and (not trigger.matchYourself or author ~= playerName) then
				-- only look them up if nil - the results are saved for any other triggers
				if isGuildMember == nil then
					if IsInGuild() then
						isGuildMember = false
						for i=1,GetNumGuildMembers() do
							if author == GetGuildRosterInfo(i)  then
								isGuildMember = true
								break
							end
						end
					end
				end
				if isFriend == nil then
					isFriend = false
					for i=1,C_FriendList.GetNumFriends() do
						if author == C_FriendList.GetFriendInfoByIndex(i).name then
							isFriend = true
							break
						end
					end
				end

				if  (trigger.matchGuild and isGuildMember)
				 or (trigger.matchFriends and isFriend)
				 or (trigger.matchOthers and not isFriend and not isGuildMember) then
					-- do nothing here since they're allowed - just easier to visualize it this way
				else
					break
				end
			end

			-- perform message modifications
			local modifiedMessage = trigger.removeCapitalization and lowercaseMessage or message
			if trigger.removePunctuation then
				modifiedMessage = modifiedMessage:gsub("%p+", ""):gsub("%s+", " ") -- also fix extra leftover spaces to be one space
			end
			if trigger.plainTextLinks then
				modifiedMessage = modifiedMessage:gsub("|c.-(%[.-])|h|r", "%1") -- convert a link to be only the [text] part
			end

			-- check phrases for a match
			local customLuaTableSet = false
			local matched = false
			if trigger.phraseMatches then
				local phraseList = trigger.phraseMatches
				for i=1,#phraseList do
					-- before matching, surround the pattern with () (except the beginning ^ end ending $) so there will always be at least one capture available
					-- customLuaTable.capture = {modifiedMessage:match(phraseList[i]:gsub("^([%^]?)(.-)([$]?)$", "%1(%2)%3"))}
					customLuaCapture[1], customLuaCapture[2], customLuaCapture[3], customLuaCapture[4], customLuaCapture[5], customLuaCapture[6],
					customLuaCapture[7], customLuaCapture[8], customLuaCapture[9], customLuaCapture[10], customLuaCapture[11], customLuaCapture[12],
					customLuaCapture[13], customLuaCapture[14], customLuaCapture[15] = modifiedMessage:match(phraseList[i]:gsub("^([%^]?)(.-)([$]?)$", "%1(%2)%3"))
					if customLuaCapture[1] ~= nil then
						matched = true
						break
					end
				end
			end

			-- check custom lua function for match
			if not matched and trigger.luaMatch then
				if not trigger.luaMatchFunc then
					local func, errorMessage = loadstring(trigger.luaMatch)
					if errorMessage then
						DEFAULT_CHAT_FRAME:AddMessage("Error in lua matching in chat trigger " .. triggerName .. ": " .. errorMessage)
						func = nil
					else
						trigger.luaMatchFunc = func
					end
				end
				if trigger.luaMatchFunc then
					for i=1,15 do
						customLuaCapture[i] = nil
					end
					customLuaTable.message = message
					customLuaTable.modifiedMessage = modifiedMessage
					customLuaTable.target = author
					customLuaTable.channel = chatChannel
					customLuaTable.channelNumber = chatChannelNumber
					customLuaTableSet = true
					matched = trigger.luaMatchFunc()
				end
			end

			-- reacting to a match
			if matched then
				if not customLuaTableSet then
					customLuaTable.message = message
					customLuaTable.modifiedMessage = modifiedMessage
					customLuaTable.target = author
					customLuaTable.channel = chatChannel
					customLuaTable.channelNumber = chatChannelNumber
				end

				-- using a message
				if trigger.useReply then
					if trigger.replyChannel == "Spell/Event" then
						AttemptReaction(trigger.useReply, 0, nil, playerName, author, RS.Common.ActionType.HIT, nil, nil, nil, nil, 1)
					else
						local reply = trigger.useReply
						local replyChannel = trigger.replyChannel
						if trigger.replyChannel == "Same Channel" then
							replyChannel = "Chat Command"
							if chatChannelNumber then
								reply = "/" .. chatChannelNumber .. " " .. reply
							elseif channelInformation[2] then
								if channelInformation[2] == SLASH_WHISPER1 then
									replyChannel = "Whisper"
								else
									reply = channelInformation[2] .. " " .. reply
								end
							end
						end
						RS.Messages.ParseAndSendMessage(reply, replyChannel, author)
					end
				end

				-- using custom lua
				if trigger.luaReaction then
					if not trigger.luaReactionFunc then
						local func, errorMessage = loadstring(trigger.luaReaction)
						if errorMessage then
							DEFAULT_CHAT_FRAME:AddMessage("Error in lua reaction in chat trigger " .. triggerName .. ": " .. errorMessage)
							func = nil
						else
							trigger.luaReactionFunc = func
						end
					end
					if trigger.luaReactionFunc then
						if ReactionsSave.testing then
							DEFAULT_CHAT_FRAME:AddMessage("|cffFFFFE0Reactions:|r Custom lua on chat trigger " .. triggerName)
						else
							trigger.luaReactionFunc()
						end
					end
				end

				-- set last used times since the message was successful
				if trigger.globalCooldown then
					trigger.lastUsed = time()
				end
				if trigger.personCooldown and author then
					if trigger.personCooldownList == nil then
						trigger.personCooldownList = {}
					end
					trigger.personCooldownList[author] = time()
				end

				-- cancel checking other triggers if wanted
				if trigger.stopTriggers then
					return
				end
			end -- if matched then
		until true -- repeat
	end -- for i=1,#channelTriggers do
end
